@@ -1,7 +1,6 @@
 # -*- coding:utf-8 -*-
 import logging
 logger = logging.getLogger(__name__)
-from operator import getitem
 from .compat import text_
 from sqlalchemy.inspection import inspect
 from sqlalchemy.orm.relationships import RelationshipProperty
@@ -11,6 +10,7 @@ from .custom.format import (
     parse_time,  # more strict than isodate
     parse_date   # more strict
 )
+from collections import defaultdict
 from jsonschema import validate
 from . import InvalidStatus
 import pytz
@@ -65,17 +65,17 @@ normalize_dict = {('string', None): maybe_wrap(text_),
 def jsonify_of(ob, name, type_, registry=jsonify_dict):
     try:
         convert_fn = registry[type_]
-        return convert_fn(getattr(ob, name))
     except KeyError:
         raise Exception("convert {} failure. unknown format {} of {}".format(name, type_, ob))
+    return convert_fn(getattr(ob, name, None))
 
 
 def normalize_of(ob, name, type_, registry=normalize_dict):
     try:
         convert_fn = registry[type_]
-        return convert_fn(ob[name])
     except KeyError:
         raise Exception("convert {} failure. unknown format {} of {}".format(name, type_, ob))
+    return convert_fn(ob.get(name))
 
 
 def attribute_of(ob, name, type_):
@@ -91,12 +91,14 @@ def jsonify(ob, schema, convert=jsonify_of, getter=getattr, registry=jsonify_dic
     return dictify_properties(ob, schema["properties"], convert=convert, getter=getter)
 
 
-def normalize(ob, schema, convert=normalize_of, getter=getitem, registry=normalize_dict):
+def normalize(ob, schema, convert=normalize_of, getter=dict.get, registry=normalize_dict):
     convert = partial(convert, registry=registry)
     return dictify_properties(ob, schema["properties"], convert=convert, getter=getter)
 
 
 def dictify_properties(ob, properties, convert, getter):
+    if ob is None:
+        return None
     D = {}
     for k, v in properties.items():
         D[k] = _dictify(ob, k, v, convert, getter)
@@ -104,9 +106,11 @@ def dictify_properties(ob, properties, convert, getter):
 
 
 def _dictify(ob, name, schema, convert, getter):
-    type_ = schema["type"]
+    type_ = schema.get("type")
     if type_ == "array":
-        return [dictify_properties(e, schema["items"], convert, getter) for e in getter(ob, name)]
+        return [dictify_properties(e, schema["items"], convert, getter) for e in getter(ob, name, [])]
+    elif type_ is None:
+        return dictify_properties(getter(ob, name), schema, convert, getter)
     elif type_ == "object":
         return dictify_properties(getter(ob, name), schema["properties"], convert, getter)
     else:
@@ -171,11 +175,17 @@ def objectify_propperties(params, properties, modellookup):
 
 def _objectify(params, name, schema, modellookup):
     type_ = schema.get("type")
+    if params is None:
+        return [] if type_ == "array" else None  # xxx
+
     if type_ == "array":
-        return [_objectify_subobject(e, name, schema["items"], modellookup) for e in params[name]]
+        return [_objectify_subobject(e, name, schema["items"], modellookup) for e in params.get(name, [])]
     elif type_ is None:  # object
+        sub_params = params.get(name)
+        if sub_params is None:
+            return None
         submodel = modellookup(name)
-        result = submodel(**{k: _objectify(params[name], k, schema[k], modellookup) for k in schema})
+        result = submodel(**{k: _objectify(params[name], k, schema[k], modellookup) for k in schema if name in params})
         modellookup.pop()
         return result
     else:
@@ -196,7 +206,7 @@ class ErrorFound(Exception):  # xxx:
 def validate_all(data, validator):
     errors = []
     for e in validator.iter_errors(data):
-        errors.append(dict(name=e.path[0], reason=e.validator))
+        errors.append(e)
     if errors:
         raise ErrorFound(errors)
     return data
